@@ -30,6 +30,7 @@ from geneticengine.grammars.sgp import Var
 from geneticengine.metahandlers.vars import VarRange
 from geneticengine.metrics import f1_score
 from geneticengine.metrics import mse
+from geneticengine.metrics import r2
 from geneticengine.off_the_shelf.sympy_compatible import fix_all
 
 
@@ -60,18 +61,21 @@ class GeneticProgrammingRegressor(BaseEstimator, TransformerMixin):
         nodes: list[type[Number]] = None,
         representation: Representation = treebased_representation,
         population_size: int = 200,
-        n_elites: int = 5,  # Shouldn't this be a percentage of population size?
+        n_elites: int = 5,
         n_novelties: int = 10,
         number_of_generations: int = 100,
         max_depth: int = 15,
         favor_less_deep_trees: bool = True,
         hill_climbing: bool = False,
         seed: int = 123,
+        metric: str = "mse",
         # -----
         # As given in A Field Guide to GP, p.17, by Poli and Mcphee
         probability_mutation: float = 0.01,
         probability_crossover: float = 0.9,
         # -----
+        timer_stop_criteria: bool = False,
+        timer_limit: int = 60,
     ):
         if nodes is None:
             nodes = [
@@ -102,6 +106,14 @@ class GeneticProgrammingRegressor(BaseEstimator, TransformerMixin):
         self.number_of_generations = number_of_generations
         self.probability_mutation = probability_mutation
         self.probability_crossover = probability_crossover
+        self.timer_stop_criteria = timer_stop_criteria
+        self.timer_limit = timer_limit
+        if metric == "r2":
+            self.metric = r2
+            self.minimise = False
+        else:
+            self.metric = mse
+            self.minimise = True
 
     def fit(self, X, y):
         """
@@ -136,10 +148,10 @@ class GeneticProgrammingRegressor(BaseEstimator, TransformerMixin):
                 variables[x] = data[:, i]
             y_pred = n.evaluate(**variables)
 
-            fitness = mse(
+            fitness = self.metric(
                 y_pred,
-                y,
-            )  # mse is used in PonyGE, as the error metric is not None!
+                target,
+            )
             if isinf(fitness) or np.isnan(fitness):
                 fitness = 100000000
             return fitness
@@ -147,7 +159,7 @@ class GeneticProgrammingRegressor(BaseEstimator, TransformerMixin):
         model = GP(
             grammar=self.grammar,
             evaluation_function=fitness_function,
-            minimize=True,
+            minimize=self.minimise,
             representation=self.representation,
             population_size=self.population_size,
             n_elites=self.n_elites,
@@ -159,9 +171,11 @@ class GeneticProgrammingRegressor(BaseEstimator, TransformerMixin):
             probability_crossover=self.probability_crossover,
             hill_climbing=self.hill_climbing,
             seed=self.seed,
+            timer_stop_criteria=self.timer_stop_criteria,
+            timer_limit=self.timer_limit,
         )
 
-        best_ind, fitness, phenotype = model.evolve(verbose=1)
+        best_ind, fitness, phenotype = model.evolve(verbose=0)
         self.evolved_phenotype = phenotype
         self.sympy_compatible_phenotype = fix_all(str(phenotype))
 
@@ -186,6 +200,11 @@ class GeneticProgrammingRegressor(BaseEstimator, TransformerMixin):
 
         return y_pred
 
+    def score(self, X, y):
+        y_pred = self.predict(X)
+
+        return r2(y_pred, y)
+
 
 class HillClimbingRegressor(BaseEstimator, TransformerMixin):
     """
@@ -217,6 +236,7 @@ class HillClimbingRegressor(BaseEstimator, TransformerMixin):
         number_of_generations: int = 100,
         max_depth: int = 15,
         seed: int = 123,
+        metric: str = "mse",
     ):
         assert Var in nodes
 
@@ -229,6 +249,12 @@ class HillClimbingRegressor(BaseEstimator, TransformerMixin):
         self.population_size = population_size
         self.max_depth = max_depth
         self.number_of_generations = number_of_generations
+        if metric == "r2":
+            self.metric = r2
+            self.minimise = False
+        else:
+            self.metric = mse
+            self.minimise = True
 
     def fit(self, X, y):
         """
@@ -263,7 +289,10 @@ class HillClimbingRegressor(BaseEstimator, TransformerMixin):
                 variables[x] = data[:, i]
             y_pred = n.evaluate(**variables)
 
-            fitness = mse(y_pred, y)
+            fitness = self.metric(
+                y_pred,
+                target,
+            )
             if isinf(fitness) or np.isnan(fitness):
                 fitness = 100000000
             return fitness
@@ -271,7 +300,7 @@ class HillClimbingRegressor(BaseEstimator, TransformerMixin):
         model = HC(
             g=self.grammar,
             evaluation_function=fitness_function,
-            minimize=True,
+            minimize=self.minimise,
             representation=self.representation,
             population_size=self.population_size,
             number_of_generations=self.number_of_generations,
@@ -279,7 +308,7 @@ class HillClimbingRegressor(BaseEstimator, TransformerMixin):
             seed=self.seed,
         )
 
-        best_ind, fitness, phenotype = model.evolve(verbose=1)
+        best_ind, fitness, phenotype = model.evolve(verbose=0)
         self.evolved_phenotype = phenotype
         self.sympy_compatible_phenotype = fix_all(str(phenotype))
 
@@ -303,4 +332,3 @@ class HillClimbingRegressor(BaseEstimator, TransformerMixin):
         y_pred = self.evolved_phenotype.evaluate(**variables)
 
         return y_pred
-
